import argparse
from unittest.mock import patch

import pytest

from jellyash.duration import average_duration, calculate_duration

from .conftest import ClientTest


class TestDuration(ClientTest):
    @pytest.mark.vcr
    def test_calculate_duration_not_found(self):
        result = calculate_duration(self.test_client, "NotFound")
        self.assertEqual(result, "NotFound not found")

    @pytest.mark.vcr
    def test_calculate_duration(self):
        result = calculate_duration(self.test_client, "Pioneer One")
        expected = "Average duration over 2 episodes: 5.4 minutes"
        self.assertEqual(result, expected)

    @pytest.mark.block_network
    def test_integration(self):
        with patch("jellyash.duration.authed_client") as client_mock:
            with patch(
                "argparse.ArgumentParser.parse_args",
                return_value=argparse.Namespace(show=["Foo", "Bar"])
            ):
                with patch(
                    "jellyash.duration.calculate_duration") as avg_mock:
                    average_duration()
                    client_mock.assert_called_once()
                    avg_mock.assert_called_once_with(
                        client_mock(), "Foo Bar"
                    )




import argparse
import unittest
from unittest.mock import patch

import pytest

from jellyash.unwatched import (
    all_unwatched,
    pluralized_str,
    specific_unwatched,
    unwatched,
)

from .conftest import ClientTest


class TestPluralizedStr(unittest.TestCase):
    def test_pluralized_str(self):
        self.assertEqual("0 unwatched episodes", pluralized_str(0))
        self.assertEqual("1 unwatched episode", pluralized_str(1))
        self.assertEqual("2 unwatched episodes", pluralized_str(2))
        self.assertEqual(
            "1 dewatched episode", pluralized_str(1, prefix="de")
        )


class TestUnwatched(ClientTest):
    def setUp(self):
        self.show = "Pioneer One"

    @pytest.fixture(autouse=True)
    def capsys(self, capsys):
        self.capsys = capsys

    @pytest.mark.vcr
    def test_all_unwatched(self):
        all_unwatched(self.test_client)
        captured = self.capsys.readouterr()
        ustr = "unwatched episode\n"
        expected = f"{self.show}: 1 {ustr}Total: 1 {ustr}"
        self.assertEqual(captured.out, expected)
        self.assertEqual(captured.err, "")

    @pytest.mark.vcr
    def test_specific_unwatched(self):
        specific_unwatched(self.test_client, "Pioneer One")
        captured = self.capsys.readouterr()
        wstr = "watched episode\n"
        expected = f"{self.show}: 1 {wstr}{self.show}: 1 un{wstr}"
        self.assertEqual(captured.out, expected)
        self.assertEqual(captured.err, "")

    @pytest.mark.vcr
    def test_specific_unwatched_not_found(self):
        specific_unwatched(self.test_client, "NotFound")
        captured = self.capsys.readouterr()
        self.assertEqual(captured.out, "NotFound not found\n")
        self.assertEqual(captured.err, "")


class TestUnwatchedIntegration(unittest.TestCase):
    @pytest.mark.block_network
    def test_unwatched_all(self):
        with patch("jellyash.unwatched.authed_client") as client_mock:
            with patch("jellyash.unwatched.all_unwatched") as all_mock:
                with patch(
                    "argparse.ArgumentParser.parse_args",
                    return_value=argparse.Namespace(show=[])
                ):
                    unwatched()
                    all_mock.assert_called_once_with(client_mock())

    @pytest.mark.block_network
    def test_unwatched_specific(self):
        with patch("jellyash.unwatched.authed_client") as client_mock:
            with patch(
                "jellyash.unwatched.specific_unwatched") as specific_mock:
                with patch(
                    "argparse.ArgumentParser.parse_args",
                    return_value=argparse.Namespace(show=["Foo", "Bar"])
                ):
                    unwatched()
                    specific_mock.assert_called_once_with(
                        client_mock(), "Foo Bar"
                    )




import unittest
from pathlib import Path

import pytest
import vcr

from jellyash.client import auth_with_password, create_client


def test_client(klass):
    dir_name = klass.__module__.split(".")[1]
    cassette_filename = f"{klass.__name__}.client.yaml"
    cassette = Path(f"tests/cassettes/{dir_name}/{cassette_filename}")
    if klass.is_recording:
        # vcrpy will not overwrite an existing cassette, remove it.
        cassette.unlink(missing_ok=True)
    with vcr.use_cassette(cassette, record_mode="once"):
        client = create_client(f"jellyash_test_{klass.__name__.lower()}")
        server_url = "https://demo.jellyfin.org/stable"
        auth_result = auth_with_password(client, server_url, "demo", "")
        assert "AccessToken" in auth_result
        return client


@pytest.fixture(scope="class")
def is_recording(request):
    is_recording = request.config.getoption("--record-mode") is not None
    request.cls.is_recording = is_recording


@pytest.mark.usefixtures("is_recording")
class ClientTest(unittest.TestCase):
    @pytest.fixture(autouse=True)
    def _client(self):
        self.test_client = test_client(self.__class__)




import inspect
import json
import pathlib
import platform
import sys
from json.decoder import JSONDecodeError
from typing import Optional
from uuid import uuid4

from jellyfin_apiclient_python.client import JellyfinClient
from jellyfin_apiclient_python.connection_manager import CONNECTION_STATE

from . import __version__
from .bundle import WrappedAPI

CREDENTIALS_FILE = pathlib.Path.home() / ".jellyfin_creds"


def create_client(app_name: Optional[str] = None):
    client = JellyfinClient()
    if not app_name:
        app_name = determine_app_name()
    client.config.app(app_name, __version__, platform.node(), str(uuid4()))
    client.config.data["auth.ssl"] = True
    client.jellyfin = WrappedAPI(client.http)
    return client


def auth_with_password(client, address: str, user: str, password: str):
    client.auth.connect_to_address(address)
    return client.auth.login(address, user, password)


def auth_with_token(client) -> None:
    if not CREDENTIALS_FILE.is_file():
        raise ValueError(f"{sys.argv[0]}: Requires credential file.")
    with open(CREDENTIALS_FILE) as f:
        credentials = json.load(f)
    state = client.authenticate({"Servers": [credentials]}, discover=False)
    if state["State"] != CONNECTION_STATE["SignedIn"]:
        raise ConnectionError("Failed to establish connection")


def authed_client():
    client = create_client(None)
    try:
        auth_with_token(client)
    except (
        PermissionError, ValueError, JSONDecodeError, ConnectionError
        ) as e:
        print(f"{sys.argv[0]}: {e}")
        sys.exit(1)
    return client


def determine_app_name() -> str:
    frame = inspect.stack()[-2]
    return f"jellyfin_{inspect.getmodulename(frame.filename)}"




from decimal import Decimal

from .cli import argparse_parser
from .client import authed_client
from .search import search_single_show


def calculate_duration(client, show: str) -> str:
    try:
        show = search_single_show(client, show)
    except ValueError as e:
        return str(e)
    count = 0
    duration = 0
    for season in client.jellyfin.get_seasons(show.Id):
        count += season.ChildCount
        episodes = client.jellyfin.get_season(show.Id, season.Id)
        duration += sum([e.RunTimeTicks for e in episodes])
    average = (duration / count) / int(Decimal("6E+008"))
    return f"Average duration over {count} episodes: {average:.1f} minutes"


def average_duration() -> None:
    parser = argparse_parser()
    parser.add_argument("show", nargs="+")
    args = parser.parse_args()
    client = authed_client()
    print(calculate_duration(client, " ".join(args.show)))



import argparse
import unittest
from collections import namedtuple
from unittest.mock import Mock, patch

import pytest

from jellyash.bundle import Item
from jellyash.nextup import episode_str, nextup

from .conftest import ClientTest


class TestEpisodeStr(ClientTest):
    @pytest.mark.vcr
    def test_episode_str(self):
        result = self.test_client.jellyfin.search_media_items(
            "The Man From Mars", media="Episodes"
        )
        episode = next((e for e in result if e.Type == "Episode"), None)
        self.assertIsNotNone(episode)
        self.assertEqual(
            episode_str(episode), "Pioneer One [1x02] The Man From Mars"
        )


class TestNextUp(unittest.TestCase):
    @pytest.fixture(autouse=True)
    def capsys(self, capsys):
        self.capsys = capsys

    @pytest.mark.block_network
    def test_nextup(self):
        client_type = namedtuple('client', ['jellyfin'])
        client = client_type(Mock())
        episode_dict = {
            'Name': 'Bar',
            'ParentIndexNumber': '4',
            'IndexNumber': '20',
            'SeriesName': 'Foo',
            }
        client.jellyfin.get_next.return_value = [Item(episode_dict)]
        with patch("jellyash.nextup.authed_client", return_value=client):
            with patch(
                "argparse.ArgumentParser.parse_args",
                return_value=argparse.Namespace(limit=10)
            ):
                nextup()
                client.jellyfin.get_next.assert_called_once_with(limit=10)
                captured = self.capsys.readouterr()
                self.assertEqual(captured.out, "Foo [4x20] Bar\n")
                self.assertEqual(captured.err, "")




import unittest

import pytest

from jellyash.bundle import ApiResponse, Item

from .conftest import ClientTest


class TestItem(unittest.TestCase):
    def test_item(self):
        item = {"User": "demo"}
        wrapped = Item(item)
        self.assertDictEqual(wrapped._raw_item(), item)
        self.assertEqual(wrapped.User, "demo")
        with self.assertRaises(AttributeError):
            wrapped.user

    def test_nested_item(self):
        item = {"SessionInfo": {"UserName": "demo", "DeviceName": "bar"}}
        wrapped = Item(item)
        self.assertEqual(wrapped.SessionInfo.UserName, "demo")
        self.assertEqual(wrapped.SessionInfo.DeviceName, "bar")


class TestApiResponse(ClientTest):
    def setUp(self):
        self.search_result = self.test_client.jellyfin.search_media_items(
            term="", media="Movies", limit=5
        )

    @pytest.mark.default_cassette("TestApiResponse.setUp.yaml")
    @pytest.mark.vcr
    def test_attrs(self):
        str_rep = "<ApiResponse object containing 95 items>"
        self.assertEqual(str(self.search_result), str_rep)
        self.assertEqual(len(self.search_result), 95)
        self.assertCountEqual(
            self.search_result._raw_value().keys(),
            ["Items", "TotalRecordCount", "StartIndex"]
        )

    @pytest.mark.default_cassette("TestApiResponse.setUp.yaml")
    @pytest.mark.vcr
    def test_getitem(self):
        item = self.search_result[4]
        self.assertEqual(item.IndexNumber, 22)
        self.assertEqual(item.Type, "AudioBook")
        self.assertFalse(item.UserData.Played)
        self.assertEqual(item.UserData.PlayCount, 0)

    @pytest.mark.default_cassette("TestApiResponse.setUp.yaml")
    @pytest.mark.vcr
    def test_iter(self):
        names = [
            "Blender Foundation",
            "18 - The Passenger of the 'Atlanta'",
            "14 - Pickaxe and Trowel",
            "31 - Part II, Chapter 26: Bow and Arrow",
            "22 - The New Citizen of the United States"
        ]
        self.assertSequenceEqual([m.Name for m in self.search_result], names)


class TestUnwrappedApiResponse(ClientTest):
    @pytest.mark.vcr
    def test_unwrapped_dict_response(self):
        resp = self.test_client.jellyfin.try_server()
        self.assertFalse(isinstance(resp, ApiResponse))
        self.assertEqual(resp["ServerName"], "Stable Demo")

    @pytest.mark.vcr
    def test_unwrapped_bool_response(self):
        resp = self.test_client.jellyfin._get("QuickConnect/Enabled")
        self.assertFalse(resp)




from collections import defaultdict
from difflib import SequenceMatcher

from .bundle import Item


def search_single_show(client, term: str) -> Item:
    r = client.jellyfin.search_media_items(term=term, media="Series")
    if len(r) == 0:
        raise ValueError(f"{term} not found")
    else:
        matches = defaultdict(list)
        seq = SequenceMatcher(b=term)
        for series in r:
            seq.set_seq1(series.Name)
            matches[seq.ratio()].append(series)
        return matches[max(matches)][0]



import json
import pathlib
import platform
import tempfile
import unittest
from unittest.mock import Mock, patch

import pytest
from jellyfin_apiclient_python.client import JellyfinClient

from jellyash import __version__
from jellyash.client import (
    auth_with_password,
    auth_with_token,
    authed_client,
    create_client,
    determine_app_name,
)

from .conftest import ClientTest


class TestClient(unittest.TestCase):
    def setUp(self):
        self.client = create_client("test")

    def test_create_client_with_app_name(self):
        data = self.client.config.data
        self.assertEqual(data['app.name'], "test")
        self.assertEqual(data['app.device_name'], platform.node())
        self.assertEqual(data['app.version'], __version__)
        self.assertTrue(data['auth.ssl'])

    @pytest.mark.vcr
    def test_auth_with_password(self):
        server_url = "https://demo.jellyfin.org/stable"
        auth_result = auth_with_password(self.client, server_url, "demo", "")
        self.assertEqual(auth_result["User"]["Name"], "demo")
        self.assertEqual(auth_result["SessionInfo"]["UserName"], "demo")
        self.assertEqual(auth_result["SessionInfo"]["DeviceName"], "wrecked")


class TestAuthWithToken(unittest.TestCase):
    def test_auth_with_token_credential_directory(self):
        with tempfile.TemporaryDirectory() as tempdir:
            ptd = pathlib.Path(tempdir)
            with patch("jellyash.client.CREDENTIALS_FILE", ptd):
                with self.assertRaises(ValueError):
                    auth_with_token(None)

    def test_auth_with_token_unreadable_file(self):
        with tempfile.NamedTemporaryFile() as tmpfile:
            ptf = pathlib.Path(tmpfile.name)
            ptf.chmod(0o0000)
            with patch("jellyash.client.CREDENTIALS_FILE", ptf):
                with self.assertRaises(PermissionError):
                    auth_with_token(None)
            ptf.chmod(0o600)

    def test_auth_with_token_invalid_json(self):
        with tempfile.NamedTemporaryFile() as tmpfile:
            tmpfile.file.write(b"{{\n")
            tmpfile.file.flush()
            ptf = pathlib.Path(tmpfile.name)
            with patch("jellyash.client.CREDENTIALS_FILE", ptf):
                with self.assertRaises(json.decoder.JSONDecodeError):
                    auth_with_token(None)

    def test_auth_with_nonexistant_file(self):
        non_exist = pathlib.Path("/does/not/exist")
        with patch("jellyash.client.CREDENTIALS_FILE", non_exist):
            with self.assertRaises(ValueError):
                auth_with_token(None)

    def test_auth_with_state_zero(self):
        with tempfile.NamedTemporaryFile() as tmpfile:
            tmpfile.file.write(b"{\"User\": \"foo\"}\n")
            tmpfile.file.flush()
            ptf = pathlib.Path(tmpfile.name)
            with patch("jellyash.client.CREDENTIALS_FILE", ptf):
                client_mock = Mock()
                client_mock.authenticate.return_value = {"State": 0}
                with self.assertRaises(ConnectionError):
                    auth_with_token(client_mock)
                client_mock.authenticate.assert_called_once()


class TestAuthedClient(ClientTest):
    @pytest.fixture(autouse=True)
    def capsys(self, capsys):
        self.capsys = capsys

    @pytest.mark.vcr
    def test_authed_client(self):
        with tempfile.NamedTemporaryFile() as tmpfile:
            credentials = self.test_client.auth.credentials.get_credentials()
            with open(tmpfile.name, 'w') as f:
                json.dump(credentials["Servers"][0], f)
            ptf = pathlib.Path(tmpfile.name)
            with patch("jellyash.client.CREDENTIALS_FILE", ptf):
                client = authed_client()
                self.assertTrue(isinstance(client, JellyfinClient))

    @pytest.mark.vcr
    def test_authed_client_offline(self):
        with tempfile.NamedTemporaryFile() as tmpfile:
            credentials = self.test_client.auth.credentials.get_credentials()
            with open(tmpfile.name, 'w') as f:
                json.dump(credentials["Servers"][0], f)
            ptf = pathlib.Path(tmpfile.name)
            with patch("jellyash.client.CREDENTIALS_FILE", ptf):
                with patch("sys.argv", ["test_offline"]):
                    with self.assertRaises(SystemExit):
                        authed_client()
        captured = self.capsys.readouterr()
        self.assertEqual(
            captured.out, "test_offline: Failed to establish connection\n"
            )
        self.assertEqual(captured.err, "")

    def test_authed_client_non_existant_file(self):
        non_exist = pathlib.Path("/does/not/exist")
        with patch("jellyash.client.CREDENTIALS_FILE", non_exist):
            with self.assertRaises(SystemExit):
                authed_client()


class TestDetermineAppName(unittest.TestCase):
    def test_determine_app_name(self):
        self.assertEqual(determine_app_name(), "jellyfin___init__")



import argparse
import json
import pathlib
import tempfile
import unittest
from unittest.mock import patch

import pytest

from jellyash.token import create_jellyfin_token


class TestCreateJellyfinToken(unittest.TestCase):
    @pytest.fixture(autouse=True)
    def capsys(self, capsys):
        self.capsys = capsys

    @pytest.mark.vcr
    def test_create_token(self):
        server_url = "https://demo.jellyfin.org/stable"
        with patch(
            "argparse.ArgumentParser.parse_args",
            return_value=argparse.Namespace(server=server_url, user="demo")
        ):
            with tempfile.NamedTemporaryFile() as tmpfile:
                ptf = pathlib.Path(tmpfile.name)
                with patch("jellyash.token.CREDENTIALS_FILE", ptf):
                    with patch(
                        "jellyash.token.getpassword", return_value=""
                    ):
                        create_jellyfin_token()
                captured = self.capsys.readouterr()
                self.assertEqual(captured.out, "Credentials saved.\n")
                self.assertEqual(captured.err, "")
                with open(ptf) as f:
                    credentials = json.load(f)
                self.assertEqual(credentials["address"], server_url)
                self.assertIn("AccessToken", credentials)
                self.assertIn("UserId", credentials)
                self.assertEqual(credentials["Name"], "Stable Demo")




import json
import unittest
from collections import namedtuple
from unittest.mock import Mock

from jellyash.bundle import ApiResponse
from jellyash.client import authed_client
from jellyash.search import search_single_show


class TestSearch(unittest.TestCase):
    def setUp(self):
        with open(self.cassette_name(self.id().split(".")[-1])) as f:
            result = json.load(f)
        client_type = namedtuple('client', ['jellyfin'])
        self.client = client_type(Mock())
        self.client.jellyfin.search_media_items.return_value = ApiResponse(
            result
            )

    def test_term_not_found(self):
        with self.assertRaises(ValueError):
            search_single_show(self.client, "NotFound")

    def test_single_term(self):
        result = search_single_show(self.client, "Chernobyl")
        self.assertEqual(result.Name, "Chernobyl")

    def test_multiple_terms_not_exact(self):
        result = search_single_show(self.client, "NCIS: ")
        self.assertEqual(result.Name, "NCIS: Hawai'i")

    def test_multiple_terms_exact(self):
        result = search_single_show(self.client, "Maid")
        self.assertEqual(result.Name, "Maid")

    def test_multiple_terms_exact_with_space(self):
        result = search_single_show(self.client, "The Blacklist")
        self.assertEqual(result.Name, "The Blacklist")

    def cassette_name(self, test):
        return f"tests/cassettes/test_search/TestSearch.{test}.yaml"

    def _rerecord_mock_results(self):
        client = authed_client()
        test_cassettes = {
            "test_term_not_found": "NotFound",
            "test_single_term": "Chernobyl",
            "test_multiple_terms_not_exact": "NCIS: ",
            "test_multiple_terms_exact": "Maid",
            "test_multiple_terms_exact_with_space": "The Blacklist",
            }
        for test in test_cassettes:
            result = client.jellyfin.search_media_items(
                term=test_cassettes[test], media="Series"
                )
            with open(self.cassette_name(test), "w") as f:
                json.dump(result._raw_value(), f)


if __name__ == "__main__":
    TestSearch()._rerecord_mock_results()




import argparse

from . import __version__


def argparse_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--version", action="version", version=f"jellyash {__version__}")
    return parser
